@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1988 Sendmail, Inc.  All rights reserved.
+ * Copyright (c) 1998 Sendmail, Inc.  All rights reserved.
  * Copyright (c) 1983 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
  *	The Regents of the University of California.  All rights reserved.
@@ -18,10 +18,12 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)mailstats.c	8.22 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)mailstats.c	8.26 (Berkeley) 7/2/98";
 #endif /* not lint */
 
-#define NOT_SENDMAIL
+#ifndef NOT_SENDMAIL
+# define NOT_SENDMAIL
+#endif
 #include <sendmail.h>
 #include <mailstats.h>
 #include <pathnames.h>
@@ -225,14 +227,14 @@ main(argc, argv)
 	}
 
 	printf("Statistics from %s", ctime(&stat.stat_itime));
-	printf(" M msgsfr  bytes_from msgsto    bytes_to  msgsrej msgsdis%s\n",
+	printf(" M   msgsfr  bytes_from   msgsto    bytes_to  msgsrej msgsdis%s\n",
 		mnames ? "  Mailer" : "");
 	for (i = 0; i < MAXMAILERS; i++)
 	{
 		if (stat.stat_nf[i] || stat.stat_nt[i] ||
 		    stat.stat_nr[i] || stat.stat_nd[i])
 		{
-			printf("%2d %6ld %10ldK %6ld %10ldK   %6ld  %6ld", i,
+			printf("%2d %8ld %10ldK %8ld %10ldK   %6ld  %6ld", i,
 			    stat.stat_nf[i], stat.stat_bf[i],
 			    stat.stat_nt[i], stat.stat_bt[i],
 			    stat.stat_nr[i], stat.stat_nd[i]);
@@ -247,8 +249,8 @@ main(argc, argv)
 			dismsgs += stat.stat_nd[i];
 		}
 	}
-	printf("=========================================================\n");
-	printf(" T %6ld %10ldK %6ld %10ldK   %6ld  %6ld\n",
+	printf("=============================================================\n");
+	printf(" T %8ld %10ldK %8ld %10ldK   %6ld  %6ld\n",
 		frmsgs, frbytes, tomsgs, tobytes, rejmsgs, dismsgs);
 	exit(EX_OK);
 }
