@@ -1,34 +1,12 @@
 /*
+ * Copyright (c) 1998 Sendmail, Inc.  All rights reserved.
  * Copyright (c) 1988, 1993
  *	The Regents of the University of California.  All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ * By using this file, you agree to the terms and conditions set
+ * forth in the LICENSE file which can be found at the top level of
+ * the sendmail distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
  */
 
 #ifndef lint
@@ -38,7 +16,7 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)rmail.c	8.3 (Berkeley) 5/15/95";
+static char sccsid[] = "@(#)rmail.c	8.17 (Berkeley) 5/19/98";
 #endif /* not lint */
 
 /*
@@ -71,19 +49,83 @@ static char sccsid[] = "@(#)rmail.c	8.3 (Berkeley) 5/15/95";
 
 #include <ctype.h>
 #include <fcntl.h>
-#include <paths.h>
+#ifdef BSD4_4
+# define FORK vfork
+# include <paths.h>
+#else
+# define FORK fork
+# ifndef _PATH_SENDMAIL
+#  define _PATH_SENDMAIL "/usr/lib/sendmail"
+# endif
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sysexits.h>
 #include <unistd.h>
+#ifdef EX_OK
+# undef EX_OK		/* unistd.h may have another use for this */
+#endif
+#include <sysexits.h>
 
 #ifndef MAX
 # define MAX(a, b)	((a) < (b) ? (b) : (a))
 #endif
 
+#ifndef __P
+# ifdef __STDC__
+#  define __P(protos)	protos
+# else
+#  define __P(protos)	()
+#  define const
+# endif
+#endif
+
+#if defined(BSD4_4) || defined(linux) || SOLARIS >= 20600 || (SOLARIS < 10000 && SOLARIS >= 206)
+# define HASSNPRINTF	1
+#endif
+
+#if defined(IRIX64) || defined(IRIX5) || defined(IRIX6) || \
+    defined(BSD4_4) || defined(__osf__) || defined(__GNU_LIBRARY__)
+# ifndef HASSTRERROR
+#  define HASSTRERROR	1	/* has strerror(3) */
+# endif
+#endif
+
+#if !HASSTRERROR
+extern char	*strerror __P((int));
+#endif
+
+#if defined(sun) && !defined(BSD) && !defined(SOLARIS) && !defined(__svr4__) && !defined(__SVR4)
+#  define memmove(d, s, l)	(bcopy((s), (d), (l)))
+#endif
+
+#if !HASSNPRINTF
+extern int	snprintf __P((char *, size_t, const char *, ...));
+#endif /* !HASSNPRINTF */
+
+u_char	tTdvect[100];
+
 void err __P((int, const char *, ...));
 void usage __P((void));
+char *xalloc __P((int));
+
+#define newstr(s)	strcpy(xalloc(strlen(s) + 1), s)
+
+char *
+xalloc(sz)
+	register int sz;
+{
+	register char *p;
+
+	/* some systems can't handle size zero mallocs */
+	if (sz <= 0)
+		sz = 1;
+
+	p = malloc((unsigned) sz);
+	if (p == NULL)
+		err(EX_TEMPFAIL, "out of memory");
+	return (p);
+}
 
 int
 main(argc, argv)
@@ -150,7 +192,8 @@ main(argc, argv)
 		/* Use the "remote from" if it exists. */
 		for (p = addrp; (p = strchr(p + 1, 'r')) != NULL;)
 			if (!strncmp(p, "remote from ", 12)) {
-				for (t = p += 12; *t && !isspace(*t); ++t);
+				for (t = p += 12;
+				     *t && !(isascii(*t) && isspace(*t)); ++t);
 				*t = '\0';
 				if (debug)
 					(void)fprintf(stderr,
@@ -159,7 +202,7 @@ main(argc, argv)
 			}
 
 		/* Else use the string up to the last bang. */
-		if (p == NULL)
+		if (p == NULL) {
 			if (*addrp == '!')
 				err(EX_DATAERR,
 				    "bang starts address: %s", addrp);
@@ -173,17 +216,16 @@ main(argc, argv)
 				if (debug)
 					(void)fprintf(stderr, "bang: %s\n", p);
 			}
-
+		}
 		/* 'p' now points to any system string from this line. */
 		if (p != NULL) {
 			/* Nul terminate it as necessary. */
-			for (t = p; *t && !isspace(*t); ++t);
+			for (t = p; *t && !(isascii(*t) && isspace(*t)); ++t);
 			*t = '\0';
 
 			/* If the first system, copy to the from_sys string. */
 			if (from_sys == NULL) {
-				if ((from_sys = strdup(p)) == NULL)
-					err(EX_TEMPFAIL, NULL);
+				from_sys = newstr(p);
 				if (debug)
 					(void)fprintf(stderr,
 					    "from_sys: %s\n", from_sys);
@@ -209,14 +251,13 @@ main(argc, argv)
 		}
 
 		/* Save off from user's address; the last one wins. */
-		for (p = addrp; *p && !isspace(*p); ++p);
+		for (p = addrp; *p && !(isascii(*p) && isspace(*p)); ++p);
 		*p = '\0';
 		if (*addrp == '\0')
 			addrp = "<>";
 		if (from_user != NULL)
 			free(from_user);
-		if ((from_user = strdup(addrp)) == NULL)
-			err(EX_TEMPFAIL, NULL);
+		from_user = newstr(addrp);
 
 		if (debug) {
 			if (from_path != NULL)
@@ -243,14 +284,12 @@ main(argc, argv)
 			domain, from_sys, domain);
 	else
 		(void)snprintf(buf, sizeof(buf), "-p%s:%s", domain, from_sys);
-	if ((args[i++] = strdup(buf)) == NULL)
-		err(EX_TEMPFAIL, NULL);
+	args[i++] = newstr(buf);
 
 					/* Set name of ``from'' person. */
 	(void)snprintf(buf, sizeof(buf), "-f%s%s",
 	    from_path ? from_path : "", from_user);
-	if ((args[i++] = strdup(buf)) == NULL)
-		err(EX_TEMPFAIL, NULL);
+	args[i++] = newstr(buf);
 
 	/*
 	 * Don't copy arguments beginning with - as they will be
@@ -294,7 +333,7 @@ main(argc, argv)
 	if (pipe(pdes) < 0)
 		err(EX_OSERR, NULL);
 
-	switch (pid = vfork()) {
+	switch (pid = FORK()) {
 	case -1:				/* Err. */
 		err(EX_OSERR, NULL);
 	case 0:					/* Child. */
@@ -371,3 +410,21 @@ err(eval, fmt, va_alist)
 	(void)fprintf(stderr, "\n");
 	exit(eval);
 }
+
+#if !HASSTRERROR
+
+char *
+strerror(eno)
+	int eno;
+{
+	extern int sys_nerr;
+	extern char *sys_errlist[];
+	static char ebuf[60];
+
+	if (eno >= 0 && eno < sys_nerr)
+		return sys_errlist[eno];
+	(void) sprintf(ebuf, "Error %d", eno);
+	return ebuf;
+}
+
+#endif /* !HASSTRERROR */
