@@ -15,9 +15,9 @@
 
 #ifndef lint
 #ifdef DAEMON
-static char sccsid[] = "@(#)daemon.c	8.217 (Berkeley) 5/19/98 (with daemon mode)";
+static char sccsid[] = "@(#)daemon.c	8.220 (Berkeley) 6/24/98 (with daemon mode)";
 #else
-static char sccsid[] = "@(#)daemon.c	8.217 (Berkeley) 5/19/98 (without daemon mode)";
+static char sccsid[] = "@(#)daemon.c	8.220 (Berkeley) 6/24/98 (without daemon mode)";
 #endif
 #endif /* not lint */
 
@@ -402,7 +402,7 @@ getrequests(e)
 			    (outchannel = fdopen(t, "w")) == NULL)
 			{
 				syserr("cannot open SMTP server channel, fd=%d", t);
-				exit(0);
+				exit(EX_OK);
 			}
 
 			InChannel = inchannel;
@@ -416,7 +416,7 @@ getrequests(e)
 			if (!xla_host_ok(RealHostName))
 			{
 				message("421 Too many SMTP sessions for this host");
-				exit(0);
+				exit(EX_OK);
 			}
 #endif
 
@@ -888,6 +888,13 @@ gothostent:
 #endif
 
 		  default:
+			if (hp->h_length > sizeof addr.sa.sa_data)
+			{
+				syserr("makeconnection: long sa_data: family %d len %d",
+					hp->h_addrtype, hp->h_length);
+				mci_setstat(mci, EX_NOHOST, "5.1.2", NULL);
+				return EX_NOHOST;
+			}
 			bcopy(hp->h_addr,
 				addr.sa.sa_data,
 				hp->h_length);
@@ -2046,7 +2053,8 @@ hostnamebyanyaddr(sap)
 	_res.retry = saveretry;
 #endif /* NAMED_BIND */
 
-	if (hp != NULL && hp->h_name[0] != '[' && inet_addr(hp->h_name) == -1)
+	if (hp != NULL && hp->h_name[0] != '[' &&
+	    inet_addr(hp->h_name) == INADDR_NONE)
 		return denlstring((char *) hp->h_name, TRUE, TRUE);
 #if NETUNIX
 	else if (sap->sa.sa_family == AF_UNIX && sap->sunix.sun_path[0] == '\0')
