@@ -14,9 +14,9 @@
 
 #ifndef lint
 #if NAMED_BIND
-static char sccsid[] = "@(#)domain.c	8.75 (Berkeley) 5/19/98 (with name server)";
+static char sccsid[] = "@(#)domain.c	8.77 (Berkeley) 6/4/98 (with name server)";
 #else
-static char sccsid[] = "@(#)domain.c	8.75 (Berkeley) 5/19/98 (without name server)";
+static char sccsid[] = "@(#)domain.c	8.77 (Berkeley) 6/4/98 (without name server)";
 #endif
 #endif /* not lint */
 
@@ -332,7 +332,8 @@ punt:
 		if (strlen(host) >= (SIZE_T) sizeof MXHostBuf)
 		{
 			*rcode = EX_CONFIG;
-			syserr("Host name %s too long", shortenstring(host, 203));
+			syserr("Host name %s too long",
+			       shortenstring(host, MAXSHORTSTR));
 			return -1;
 		}
 		snprintf(MXHostBuf, sizeof MXHostBuf, "%s", host);
@@ -436,6 +437,7 @@ mxrand(host)
 **	to generalize it at the moment.
 */
 
+/* ARGSUSED3 */
 char *
 bestmx_map_lookup(map, name, av, statp)
 	MAP *map;
