@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)headers.c	8.126 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)headers.c	8.127 (Berkeley) 6/4/98";
 #endif /* not lint */
 
 # include <errno.h>
@@ -182,7 +182,7 @@ chompheader(line, def, hdrp, e)
 		(void) sendtolist(fvalue, NULLADDR, &e->e_errorqueue, 0, e);
 
 	/* if this means "end of header" quit now */
-	if (bitset(H_EOH, hi->hi_flags))
+	if (!headeronly && bitset(H_EOH, hi->hi_flags))
 		return hi->hi_flags;
 
 	/*
@@ -254,6 +254,10 @@ chompheader(line, def, hdrp, e)
 	bcopy((char *) mopts, (char *) h->h_mflags, sizeof mopts);
 	*hp = h;
 	h->h_flags = hi->hi_flags;
+
+	/* strip EOH flag if parsing MIME headers */
+	if (headeronly)
+		h->h_flags &= ~H_EOH;
 	if (def)
 		h->h_flags |= H_DEFAULT;
 	if (cond)
