@@ -19,7 +19,7 @@ static char copyright[] =
 #endif /* not lint */
 
 #ifndef lint
-static char sccsid[] = "@(#)main.c	8.300 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)main.c	8.302 (Berkeley) 6/4/98";
 #endif /* not lint */
 
 #define	_DEFINE
@@ -1402,7 +1402,7 @@ main(argc, argv, envp)
 			if (i < 0)
 				syserr("daemon: cannot fork");
 			if (i != 0)
-				exit(0);
+				exit(EX_OK);
 
 			/* disconnect from our controlling tty */
 			disconnect(2, CurEnv);
@@ -1621,6 +1621,7 @@ main(argc, argv, envp)
 }
 
 
+/* ARGSUSED */
 SIGFUNC_DECL
 intindebug(sig)
 	int sig;
@@ -1709,6 +1710,7 @@ finis()
 **		Unlocks the current job.
 */
 
+/* ARGSUSED */
 SIGFUNC_DECL
 intsig(sig)
 	int sig;
@@ -2136,6 +2138,7 @@ dumpstate(when)
 }
 
 
+/* ARGSUSED */
 SIGFUNC_DECL
 sigusr1(sig)
 	int sig;
@@ -2145,6 +2148,7 @@ sigusr1(sig)
 }
 
 
+/* ARGSUSED */
 SIGFUNC_DECL
 sighup(sig)
 	int sig;
