@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)parseaddr.c	8.150 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)parseaddr.c	8.153 (Berkeley) 6/24/98";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -1066,7 +1066,8 @@ rewrite(pvp, ruleset, reclevel, e)
 					char *mval = macvalue(rp[1], e);
 					char **xpvp;
 					int trsize = 0;
-					char pvpb1[MAXATOM + 1];
+					static size_t pvpb1_size = 0;
+					static char **pvpb1 = NULL;
 					char pvpbuf[PSBUFSIZE];
 
 					if (tTd(21, 2))
@@ -1079,6 +1080,14 @@ rewrite(pvp, ruleset, reclevel, e)
 					/* save the remainder of the input */
 					for (xpvp = pvp; *xpvp != NULL; xpvp++)
 						trsize += sizeof *xpvp;
+					if (trsize > pvpb1_size)
+					{
+						if (pvpb1 != NULL)
+							free(pvpb1);
+						pvpb1 = (char **)xalloc(trsize);
+						pvpb1_size = trsize;
+					}
+
 					bcopy((char *) pvp, (char *) pvpb1, trsize);
 
 					/* scan the new replacement */
@@ -1464,7 +1473,7 @@ map_lookup(map, key, argvect, pstat, e)
 			snprintf(mbuf, sizeof mbuf,
 				"%.80s map: lookup (%s): deferred",
 				map->s_name,
-				shortenstring(key, 203));
+				shortenstring(key, MAXSHORTSTR));
 			e->e_message = newstr(mbuf);
 		}
 	}
@@ -2296,6 +2305,7 @@ dequote_init(map, args)
 **		else -- The dequoted buffer.
 */
 
+/* ARGSUSED2 */
 char *
 dequote_map(map, name, av, statp)
 	MAP *map;
@@ -2454,7 +2464,7 @@ rscheck(rwset, p1, p2, e)
 			printf("rscheck: cannot prescan input\n");
 /*
 		syserr("rscheck: cannot prescan input: \"%s\"",
-			shortenstring(buf, 203));
+			shortenstring(buf, MAXSHORTSTR));
 		rstat = EX_DATAERR;
 */
 		goto finis;
