@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)readcf.c	8.229 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)readcf.c	8.230 (Berkeley) 6/5/98";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -1515,6 +1515,10 @@ struct optioninfo
 #define O_CNCTONLYTO	0xa6
 	{ "ConnectOnlyTo",		O_CNCTONLYTO,	FALSE	},
 #endif
+#if _FFR_TRUSTED_FILE_OWNER
+#define O_TRUSTFILEOWN	0xa7
+	{ "TrustedFileOwner",		O_TRUSTFILEOWN,	FALSE	},
+#endif
 
 	{ NULL,				'\0',		FALSE	}
 };
@@ -2397,6 +2401,33 @@ setoption(opt, val, safe, sticky, e)
 		break;
 #endif
 
+#if _FFR_TRUSTED_FILE_OWNER
+	  case O_TRUSTFILEOWN:
+		if (isascii(*val) && isdigit(*val))
+			TrustedFileUid = atoi(val);
+		else
+		{
+			register struct passwd *pw;
+
+			TrustedFileUid = 0;
+			pw = sm_getpwnam(val);
+			if (pw == NULL)
+				syserr("readcf: option TrustedFileOwner: unknown user %s", val);
+			else
+				TrustedFileUid = pw->pw_uid;
+		}
+
+#ifdef UID_MAX
+		if (TrustedFileUid > UID_MAX)
+		{
+			syserr("readcf: option TrustedFileOwner: uid value (%ld) > UID_MAX (%ld)",
+				TrustedFileUid, UID_MAX);
+			TrustedFileUid = 0;
+		}
+#endif
+		break;
+#endif
+
 	  default:
 		if (tTd(37, 1))
 		{
