@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)recipient.c	8.151 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)recipient.c	8.154 (Berkeley) 6/24/98";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -365,12 +365,12 @@ recipient(a, sendq, aliaslevel, e)
 				if (LogLevel > 2)
 					sm_syslog(LOG_ERR, e->e_id,
 						"include %s: transient error: %s",
-						shortenstring(a->q_user, 203),
+						shortenstring(a->q_user, MAXSHORTSTR),
 						errstring(ret));
 				a->q_flags |= QQUEUEUP;
 				a->q_flags &= ~QDONTSEND;
 				usrerr("451 Cannot open %s: %s",
-					shortenstring(a->q_user, 203),
+					shortenstring(a->q_user, MAXSHORTSTR),
 					errstring(ret));
 			}
 			else if (ret != 0)
@@ -378,7 +378,7 @@ recipient(a, sendq, aliaslevel, e)
 				a->q_flags |= QBADADDR;
 				a->q_status = "5.2.4";
 				usrerr("550 Cannot open %s: %s",
-					shortenstring(a->q_user, 203),
+					shortenstring(a->q_user, MAXSHORTSTR),
 					errstring(ret));
 			}
 		}
@@ -1045,7 +1045,8 @@ include(fname, forwarding, ctladdr, sendq, aliaslevel, e)
 			}
 			else
 				sfflags |= SFF_SAFEDIRPATH;
-			if (!bitset((forwarding ?
+			if (ret > E_PSEUDOBASE &&
+			    !bitset((forwarding ?
 				     DBS_FORWARDFILEINUNSAFEDIRPATHSAFE :
 				     DBS_INCLUDEFILEINUNSAFEDIRPATHSAFE),
 				    DontBlameSendmail))
@@ -1053,7 +1054,7 @@ include(fname, forwarding, ctladdr, sendq, aliaslevel, e)
 				if (LogLevel >= 12)
 					sm_syslog(LOG_INFO, e->e_id,
 						  "%s: unsafe directory path, marked unsafe",
-						  shortenstring(fname, 203));
+						  shortenstring(fname, MAXSHORTSTR));
 				ctladdr->q_flags |= QUNSAFEADDR;
 			}
 		}
@@ -1082,7 +1083,7 @@ include(fname, forwarding, ctladdr, sendq, aliaslevel, e)
 		if (tTd(27, 4))
 			printf("include: open: %s\n", errstring(rval));
 	}
-	else if (filechanged(fname, fileno(fp), &st, sfflags))
+	else if (filechanged(fname, fileno(fp), &st))
 	{
 		rval = E_SM_FILECHANGE;
 		if (tTd(27, 4))
@@ -1173,7 +1174,7 @@ resetuid:
 				if (LogLevel >= 12)
 					sm_syslog(LOG_INFO, e->e_id,
 						"%s: user %s has bad shell %s, marked %s",
-						shortenstring(fname, 203),
+						shortenstring(fname, MAXSHORTSTR),
 						pw->pw_name, sh,
 						safechown ? "bogus" : "unsafe");
 				if (safechown)
@@ -1212,12 +1213,12 @@ resetuid:
 	{
 		if (tTd(27, 6))
 			printf("include: %s is %s writable, marked unsafe\n",
-				shortenstring(fname, 203),
+				shortenstring(fname, MAXSHORTSTR),
 				bitset(S_IWOTH, st.st_mode) ? "world" : "group");
 		if (LogLevel >= 12)
 			sm_syslog(LOG_INFO, e->e_id,
 				"%s: %s writable %s file, marked unsafe",
-				shortenstring(fname, 203),
+				shortenstring(fname, MAXSHORTSTR),
 				bitset(S_IWOTH, st.st_mode) ? "world" : "group",
 				forwarding ? "forward" : ":include:");
 		ctladdr->q_flags |= QUNSAFEADDR;
@@ -1259,7 +1260,7 @@ resetuid:
 		if (forwarding && LogLevel > 9)
 			sm_syslog(LOG_INFO, e->e_id,
 				"forward %.200s => %s",
-				oldto, shortenstring(buf, 203));
+				oldto, shortenstring(buf, MAXSHORTSTR));
 
 		nincludes += sendtolist(buf, ctladdr, sendq, aliaslevel + 1, e);
 	}
