@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)savemail.c	8.136 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)savemail.c	8.138 (Berkeley) 6/17/98";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -345,6 +345,8 @@ savemail(e, sendbody)
 			define('z', p, e);
 			expand("\201z/dead.letter", buf, sizeof buf, e);
 			flags = SFF_CREAT|SFF_REGONLY|SFF_RUNASREALUID;
+			if (RealUid == 0)
+				flags |= SFF_ROOTOK;
 			e->e_to = buf;
 			if (mailfile(buf, FileMailer, NULL, flags, e) == EX_OK)
 			{
@@ -541,7 +543,7 @@ returntosender(msg, returnq, flags, e)
 			p = "DSN";
 		sm_syslog(LOG_INFO, e->e_id,
 			"%s: %s: %s",
-			ee->e_id, p, shortenstring(msg, 203));
+			ee->e_id, p, shortenstring(msg, MAXSHORTSTR));
 	}
 
 	if (SendMIMEErrors)
@@ -786,12 +788,13 @@ errbody(mci, e, separator)
 			printheader = FALSE;
 		}
 
-		snprintf(buf, sizeof buf, "%s", shortenstring(q->q_paddr, 203));
+		snprintf(buf, sizeof buf, "%s",
+			 shortenstring(q->q_paddr, MAXSHORTSTR));
 		putline(buf, mci);
 		if (q->q_alias != NULL)
 		{
 			snprintf(buf, sizeof buf, "    (expanded from: %s)",
-				shortenstring(q->q_alias->q_paddr, 203));
+				shortenstring(q->q_alias->q_paddr, MAXSHORTSTR));
 			putline(buf, mci);
 		}
 	}
@@ -813,12 +816,13 @@ errbody(mci, e, separator)
 			printheader = FALSE;
 		}
 
-		snprintf(buf, sizeof buf, "%s", shortenstring(q->q_paddr, 203));
+		snprintf(buf, sizeof buf, "%s",
+			 shortenstring(q->q_paddr, MAXSHORTSTR));
 		putline(buf, mci);
 		if (q->q_alias != NULL)
 		{
 			snprintf(buf, sizeof buf, "    (expanded from: %s)",
-				shortenstring(q->q_alias->q_paddr, 203));
+				shortenstring(q->q_alias->q_paddr, MAXSHORTSTR));
 			putline(buf, mci);
 		}
 	}
@@ -856,12 +860,12 @@ errbody(mci, e, separator)
 		}
 
 		snprintf(buf, sizeof buf, "%s  (%s)",
-			shortenstring(q->q_paddr, 203), p);
+			shortenstring(q->q_paddr, MAXSHORTSTR), p);
 		putline(buf, mci);
 		if (q->q_alias != NULL)
 		{
 			snprintf(buf, sizeof buf, "    (expanded from: %s)",
-				shortenstring(q->q_alias->q_paddr, 203));
+				shortenstring(q->q_alias->q_paddr, MAXSHORTSTR));
 			putline(buf, mci);
 		}
 	}
