@@ -9,7 +9,7 @@
  * the sendmail distribution.
  *
  *
- *	@(#)sendmail.h	8.278 (Berkeley) 5/19/98
+ *	@(#)sendmail.h	8.280 (Berkeley) 6/5/98
  */
 
 /*
@@ -19,7 +19,7 @@
 # ifdef _DEFINE
 # define EXTERN
 # ifndef lint
-static char SmailSccsId[] =	"@(#)sendmail.h	8.278		5/19/98";
+static char SmailSccsId[] =	"@(#)sendmail.h	8.280		6/5/98";
 # endif
 # else /*  _DEFINE */
 # define EXTERN extern
@@ -998,7 +998,7 @@ extern int	safedirpath __P((char *, UID_T, GID_T, char *, int));
 extern int	safeopen __P((char *, int, int, int));
 extern FILE	*safefopen __P((char *, int, int, int));
 extern int	dfopen __P((char *, int, int, int));
-extern bool	filechanged __P((char *, int, struct stat *, int));
+extern bool	filechanged __P((char *, int, struct stat *));
 
 
 /*
@@ -1183,6 +1183,7 @@ EXTERN gid_t	RealGid;	/* real gid of caller */
 EXTERN uid_t	DefUid;		/* default uid to run as */
 EXTERN gid_t	DefGid;		/* default gid to run as */
 EXTERN char	*DefUser;	/* default user to run as (from DefUid) */
+EXTERN uid_t	TrustedFileUid;	/* uid of trusted owner of files and dirs */
 EXTERN MODE_T	OldUmask;	/* umask when sendmail starts up */
 EXTERN int	Verbose;	/* set if blow-by-blow desired */
 EXTERN int	Errors;		/* set if errors (local to single pass) */
