@@ -14,9 +14,9 @@
 
 #ifndef lint
 #if USERDB
-static char sccsid [] = "@(#)udb.c	8.63 (Berkeley) 5/19/98 (with USERDB)";
+static char sccsid [] = "@(#)udb.c	8.66 (Berkeley) 6/18/98 (with USERDB)";
 #else
-static char sccsid [] = "@(#)udb.c	8.63 (Berkeley) 5/19/98 (without USERDB)";
+static char sccsid [] = "@(#)udb.c	8.66 (Berkeley) 6/18/98 (without USERDB)";
 #endif
 #endif
 
@@ -323,7 +323,7 @@ udbexpand(a, sendq, aliaslevel, e)
 				sm_syslog(LOG_INFO, e->e_id,
 					"expand %.100s => %s",
 					e->e_to,
-					shortenstring(user, 203));
+					shortenstring(user, MAXSHORTSTR));
 			naddrs = sendtolist(user, a, sendq, aliaslevel + 1, e);
 			if (naddrs > 0 && !bitset(QSELFREF, a->q_flags))
 			{
@@ -461,7 +461,7 @@ udbexpand(a, sendq, aliaslevel, e)
 				sm_syslog(LOG_INFO, e->e_id,
 					"hesiod %.100s => %s",
 					e->e_to,
-					shortenstring(user, 203));
+					shortenstring(user, MAXSHORTSTR));
 			naddrs = sendtolist(user, a, sendq, aliaslevel + 1, e);
 
 			if (naddrs > 0 && !bitset(QSELFREF, a->q_flags))
@@ -641,7 +641,6 @@ udbmatch(user, field)
 			if (tTd(28, 1))
 				printf("udbmatch ==> %s\n", p);
 			return p;
-			break;
 #endif
 
 #ifdef HESIOD
@@ -742,7 +741,6 @@ udbmatch(user, field)
 			if (tTd(28, 1))
 				printf("udbmatch ==> %s\n", p);
 			return p;
-			break;
 #endif
 
 #ifdef HESIOD
@@ -809,6 +807,7 @@ udbmatch(user, field)
 **		The rewritten name otherwise.
 */
 
+/* ARGSUSED3 */
 char *
 udb_map_lookup(map, name, av, statp)
 	MAP *map;
@@ -868,6 +867,7 @@ int
 _udbx_init(e)
 	ENVELOPE *e;
 {
+	int ents = 0;
 	register char *p;
 	register struct udbent *up;
 
@@ -884,7 +884,6 @@ _udbx_init(e)
 	while (p != NULL)
 	{
 		char *spec;
-		int nopts;
 		int l;
 # if 0
 		auto int rcode;
@@ -905,8 +904,14 @@ _udbx_init(e)
 		if (p != NULL)
 			*p++ = '\0';
 
+		if (ents >= MAXUDBENT)
+		{
+			syserr("Maximum number of UDB entries exceeded");
+			break;
+		}
+
 		/* extract options */
-		nopts = _udb_parsespec(spec, opts, MAXUDBOPTS);
+		(void) _udb_parsespec(spec, opts, MAXUDBOPTS);
 
 		/*
 		**  Decode database specification.
@@ -973,6 +978,7 @@ _udbx_init(e)
 				      INADDRSZ);
 				up->udb_addr.sin_port = UdbPort;
 				up->udb_timeout = UdbTimeout;
+				ents++;
 				up++;
 			}
 
@@ -988,6 +994,7 @@ _udbx_init(e)
 		  case '@':	/* forward to remote host */
 			up->udb_type = UDB_FORWARD;
 			up->udb_fwdhost = spec + 1;
+			ents++;
 			up++;
 			break;
 
@@ -997,6 +1004,7 @@ _udbx_init(e)
 			if (strcasecmp(spec, "hesiod") != 0)
 				goto badspec;
 			up->udb_type = UDB_HESIOD;
+			ents++;
 			up++;
 			break;
 #endif /* HESIOD */
@@ -1059,6 +1067,7 @@ _udbx_init(e)
 				break;
 			}
 			up->udb_type = UDB_DBFETCH;
+			ents++;
 			up++;
 			break;
 #endif
