@@ -14,9 +14,9 @@
 
 #ifndef lint
 #if SMTP
-static char sccsid[] = "@(#)usersmtp.c	8.102 (Berkeley) 5/19/98 (with SMTP)";
+static char sccsid[] = "@(#)usersmtp.c	8.104 (Berkeley) 6/30/98 (with SMTP)";
 #else
-static char sccsid[] = "@(#)usersmtp.c	8.102 (Berkeley) 5/19/98 (without SMTP)";
+static char sccsid[] = "@(#)usersmtp.c	8.104 (Berkeley) 6/30/98 (without SMTP)";
 #endif
 #endif /* not lint */
 
@@ -480,7 +480,7 @@ smtpmailfrom(m, mci, e)
 	else if (r == 552)
 	{
 		/* exceeded storage allocation */
-		mci_setstat(mci, EX_NOTSTICKY, "5.2.2", SmtpReplyBuffer);
+		mci_setstat(mci, EX_NOTSTICKY, "5.3.4", SmtpReplyBuffer);
 		if (bitset(MCIF_SIZE, mci->mci_flags))
 			e->e_flags |= EF_NO_BODY_RETN;
 		return EX_UNAVAILABLE;
@@ -1026,7 +1026,7 @@ reply(m, mci, e, timeout, pfunc)
 					int plen;
 
 					snprintf(p, wbufleft, "%s... ",
-						shortenstring(e->e_to, 203));
+						shortenstring(e->e_to, MAXSHORTSTR));
 					plen = strlen(p);
 					p += plen;
 					wbufleft -= plen;
