@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)util.c	8.156 (Berkeley) 5/19/98";
+static char sccsid[] = "@(#)util.c	8.159 (Berkeley) 7/1/98";
 #endif /* not lint */
 
 # include "sendmail.h"
@@ -386,6 +386,8 @@ xputs(s)
 				if (c == MACRODEXPAND)
 					putchar('&');
 				shiftout = TRUE;
+				if (*s == '\0')
+					continue;
 				if (strchr("=~&?", *s) != NULL)
 					putchar(*s++);
 				if (bitset(0200, *s))
@@ -922,6 +924,7 @@ sfgets(buf, siz, fp, timeout, during)
 	return (buf);
 }
 
+/* ARGSUSED */
 static void
 readtimeout(timeout)
 	time_t timeout;
@@ -1778,7 +1781,7 @@ denlstring(s, strict, logattacks)
 		sm_syslog(LOG_NOTICE, CurEnv->e_id,
 			"POSSIBLE ATTACK from %.100s: newline in string \"%s\"",
 			RealHostName == NULL ? "[UNKNOWN]" : RealHostName,
-			shortenstring(bp, 203));
+			shortenstring(bp, MAXSHORTSTR));
 	}
 
 	return bp;
